/* 
 *  Author:     Alan Taran
 *  Class:      JavaScript 2
 *  Project:    Final Project
 *  Date:       March 2016
 *  Remarks:    Heat Map Script
 *              - capture mouse click activity
 *              - capture mouse movement (user search) activity
 *                obtaining the time duration mouse remained in area
 *              - send activity data to server via ajax post
 *              - retrive activity, format data, and show heat map
 *              Normal Page Operations
 *              - get json data via ajax get
 */


/*
 *  global variables 
 *  
 */
var clickStorageKey = "UserClick";
var moveStorageKey = "UserMove";
var timer;

var clickHeatmapData;
var moveHeatmapData;

//  store click and mouse move to position within grid blocks
//  NOTE:   Changing this value without refreshing the data in the backend server
//          table (or converting the data) will cause heat map to render incorrectly.
var gridAreaSize = 9;          

//  page section grid positions
var navLeft;
var navRight;
var navTop;
var navBottom;
var footerLeft;
var footerRight;
var footerTop;
var footerBottom;



/*
 *  document ready 
 *      (page fully loaded, ready for manipulation)
 *      prepare for normal page operations
 */		
$(document).ready(function() {
    pageGridSections();

    // page event handlers
    $("#quotenorm").on("click", getQuote);
    $("#quoteclavin").on("click", getQuote);
    $("#quotehomer").on("click", getQuote);
    $("#quoteberra").on("click", getQuote);
    $("#quoteholts").on("click", getQuote);
    $("#quotelombardi").on("click", getQuote);
    $("#quoteeinstein").on("click", getQuote);
    $("#quotestevejobs").on("click", getQuote);
    $("#quotetechfunny").on("click", getQuote);
    $(window).on("resize", pageGridSections);

    // heat map event handlers
    $("#wrapper").on("click", captureMouseClick);
    $("#wrapper").on("mousemove", captureMouseMovement); 
        
    $("#heatmapbtn").on("click", viewHeatMap);
    // ensure button is available after user refresh
    $("#heatmapbtn").attr("disabled", false);     
    
    // send captured user activity to backend every 10 seconds
    timer = setInterval(sendDataToServer, 10000);
});



/*
 *  onbeforeunload
 *      send remaining user activity to backend for storage before leaving page
 */
window.onbeforeunload = function() {
    // log time duration of pending last entry
    if (sessionStorage.getItem(moveStorageKey) !== null) {
        var time = new Date().getTime();
        var moveData = JSON.parse(sessionStorage.getItem(moveStorageKey));
        if (moveData.length > 0 && moveData[moveData.length - 1].time > 1000000) {
            var duration = time - moveData[moveData.length - 1].time;
            if (duration > 99) {
                moveData[moveData.length - 1].time = duration;
                var jsonString = JSON.stringify(moveData);
                sessionStorage.setItem(moveStorageKey, jsonString);
            } else {
                moveData.pop();
            }
        }
    }
    // send all remaining activity to server
    sendDataToServer();
    sessionStorage.removeItem(moveStorageKey);
};




// -----------------------------------------------------------------------------
//
//      Normal Page (website) Functions
//
//  ----------------------------------------------------------------------------

/*
 *  getQuote
 *      retrieve the selected quote (click on author's name event)
 */
function getQuote(event) {
    var quoteUrl = "data/";
    switch (event.target.id) {
        case "quotenorm" :
            quoteUrl += "normquotes";
            break;
        case "quoteclavin" :
            quoteUrl += "cliffclavinquotes";
            break;
        case "quotehomer" :
            quoteUrl += "homersimpsonquotes";
            break;
        case "quoteberra" :
            quoteUrl += "yogiberraquotes";
            break;
        case "quoteholts" :
            quoteUrl += "louholtsquotes";
            break;
        case "quotelombardi" :
            quoteUrl += "vincelombardiquotes";
            break;
        case "quoteeinstein" :
            quoteUrl += "alberteinsteinquotes";
            break;
        case "quotestevejobs" :
            quoteUrl += "stevejobsquotes";
            break;
        case "quotetechfunny" :
            quoteUrl += "techfunnyquotes";
            break;
    }    
    quoteUrl += ".json";
    
    $.ajax({
        url: quoteUrl,
        type: "get",
        dataType: "text",
        success: renderQuote,
        error: 
            function(exc) {
                msg = "Error trying to get quote: " + exc.message;
                console.log(msg);
                alert(msg);
        }
    });
}


/*
 *  renderQuote
 *      format and display quotes for the selected author
 */
function renderQuote(data) {
    var returnData = JSON.parse(data);
    $("#author").html(returnData.author);
    $("#quotedsayings").html("");
    for(var q in returnData.sayings) {
        $("#quotedsayings").append("<li>");
        for(var ln in returnData.sayings[q].quote) {
            $("#quotedsayings").append(returnData.sayings[q].quote[ln].line + "<br>");
        }
        $("#quotedsayings").append("</li>");
    }
}




//  ----------------------------------------------------------------------------
//
//      Capture User Activity for Heat Map 
//
//  ----------------------------------------------------------------------------


/*
 *  captureMouseClick
 *      store position where user mouse clicked on the page
 */
function captureMouseClick(event) {
    var clickArray = new Array(); 
    if (sessionStorage.getItem(clickStorageKey) !== null) {
        clickArray = JSON.parse(sessionStorage.getItem(clickStorageKey));
    }
    
    // determine which page section grid user clicked in
    // then accumulate data in the appropriate grid block of that section
    if (event.pageY >= navTop && event.pageY <= navBottom &&
            event.pageX >= navLeft && event.pageX <= navRight) {
        clickArray.push({ 
            "grid" : "nav", 
            "X" : Math.ceil((event.pageX - navLeft) / gridAreaSize), 
            "Y" : Math.ceil(event.pageY / gridAreaSize)
        });
    } else if (event.pageY >= footerTop && event.pageY <= footerBottom &&
                   event.pageX >= footerLeft && event.pageX <= footerRight) {
        clickArray.push({ 
            "grid" : "ftr", 
            "X" : Math.ceil((event.pageX - footerLeft) / gridAreaSize),
            "Y" : Math.ceil(event.pageY / gridAreaSize)
        });
    } else {
        clickArray.push({ 
            "grid" : "pg", 
            "X" : Math.ceil(event.pageX / gridAreaSize), 
            "Y" : Math.ceil(event.pageY / gridAreaSize) 
        });
    }
    
    // store event info
    var jsonString = JSON.stringify(clickArray);
    sessionStorage.setItem(clickStorageKey, jsonString);
}



/*
 *  captureMouseMovement
 *      finalize previous mousemove by determining the time duration,
 *      store area where user moved the mouse cursor
 */
function captureMouseMovement(event) {
    var currTime = new Date().getTime();

    var moveArray = new Array();
    if (sessionStorage.getItem(moveStorageKey) !== null) {
        moveArray = JSON.parse(sessionStorage.getItem(moveStorageKey));
        // set the duration mouse stayed in page location to prior mousemove position
        if (moveArray.length > 0) {
            var duration =   currTime - moveArray[moveArray.length - 1].time;
            // retain when duration is greater than 1/10 of a second
            if (duration > 99) {
                moveArray[moveArray.length - 1].time = duration;
            } else {
                moveArray.pop();
            }
        }        
    }
   
    // Determine which page section grid mouse was searching in.
    // Then accumulate data in the appropriate grid block of that section.
    if (event.pageY >= navTop && event.pageY <= navBottom &&
            event.pageX >= navLeft && event.pageX <= navRight) {
        moveArray.push({ 
            "grid" : "nav", 
            "X" : Math.ceil((event.pageX - navLeft) / gridAreaSize), 
            "Y" : Math.ceil(event.pageY / gridAreaSize),
            "time" : currTime 
        });
    } else if (event.pageY >= footerTop && event.pageY <= footerBottom &&
                   event.pageX >= footerLeft && event.pageX <= footerRight) {
        moveArray.push({ 
            "grid" : "ftr", 
            "X" : Math.ceil((event.pageX - footerLeft) / gridAreaSize),
            "Y" : Math.ceil(event.pageY / gridAreaSize),
            "time" : currTime 
        });
    } else { 
        moveArray.push({ 
            "grid" : "pg", 
            "X" : Math.ceil(event.pageX / gridAreaSize), 
            "Y" : Math.ceil(event.pageY / gridAreaSize),
            "time" : currTime 
        });
    } 
    
    // store event info
    if (moveArray.length > 0) {
        var jsonString = JSON.stringify(moveArray);
        sessionStorage.setItem(moveStorageKey, jsonString);
    }
}



/*
 *  sendDataToServer
 *      send captured user activity to backend server
 *      clear user activity data to prevent duplicate entries 
 */
function sendDataToServer() {
    var jsonRequest;
    var sendRequest = new Object();
    
    // click activity
    if (sessionStorage.getItem(clickStorageKey) !== null) {
        var clickData = JSON.parse(sessionStorage.getItem(clickStorageKey));
        sessionStorage.removeItem(clickStorageKey);
        
        sendRequest.event = 'click';
        sendRequest.data = clickData;
        jsonRequest = JSON.stringify(sendRequest);
       
        // ajax post
        $.ajax({
            url: "heatmapstore/storeactivity.php",
            type: "post",
            dataType: "json",
            data: { "userActivity" : jsonRequest },
            success:
                function(data) {
                    console.log("add click activity return msg data :" + data);
                    for (var i = 0; i < data.length; i++) {
                        if (data[i] !== "success") {
                            console.log("Add CLICK activity msg: " + data[i]); 
                        }
                    }
                },
            error:
                function(exc) {
                    msg = "Add CLICK activity Error: " + exc.message;
                    console.log(msg);
                }
        });        
    }

    // mousemove (user searching) activity
    if (sessionStorage.getItem(moveStorageKey) !== null) {
        var moveData = JSON.parse(sessionStorage.getItem(moveStorageKey));
        sessionStorage.removeItem(moveStorageKey);

        // retain last entry if mouse is still remaining in an area, duration pending
        if (moveData.length > 0 && moveData[moveData.length - 1].time > 100000000) {
            var arrItem = new Array();
            arrItem.push(moveData.pop());
            var arrString = JSON.stringify(arrItem);
            sessionStorage.setItem(moveStorageKey, arrString);
        }
        
        if (moveData.length > 0) {
            sendRequest.event = 'move';
            sendRequest.data = moveData;
            jsonRequest = JSON.stringify(sendRequest);

            // ajax post
            $.ajax({
                url: "heatmapstore/storeactivity.php",
                type: "post",
                dataType: "json",
                data: { "userActivity" : jsonRequest },
                success:
                    function(data) {  
                        console.log("add move activity return msg data :" + data);
                        for (var i = 0; i < data.length; i++) {
                            if (data[i] !== "success") {
                                console.log("Add MOVE activity msg: " + data[i]); 
                            }
                        }
                    },
                error:
                    function(exc) {
                        msg = "Add MOVE activity Error: " + exc.message;
                        console.log(msg);
                    }
            });
        } 
    } 
}




//  ----------------------------------------------------------------------------
//
//      Generate Heat Map functions
//
//  ----------------------------------------------------------------------------


/*
 *  viewHeatMap
 *      stop normal operations
 *      retrieve user activity from server 
 *      analyze the data and format for map rendering
 *      render the heat map
 */
function viewHeatMap(event) {
    // turn off normal page operations 
    $("#wrapper").off("mousemove");
    $("#wrapper").off("click");
    clearInterval(timer);    
    $(window).off("resize");
    $("#heatmapbtn").attr("disabled", true);
    
    // capture the click of the view heat map button (not required to capture normal page activity)
    captureMouseClick(event);
        
    //  send data to server to store activity since last send
    sendDataToServer();

    preparePageforMap();
    
    //  get user activity from database
    var dataString = "";    
    $.ajax({
        url: "heatmapstore/getactivity.php",
        type: "get",
        dataType: "json",
        data: dataString,
        success:
            function(data) {
                formatData(data);
                loadHeatMap();
            },
        error:
            function(exc) {
                msg = "get user activity Error: " + exc.message;
                console.log(msg);
            }
    });
}



/*
 *  formatData
 *      analize user activity and build formatted data for heat maps
 *      - determine the color value based on each user activity event's duration
 *      - mouse click events:       duration is the number of times an area has been clicked
 *      - mousemove search events:  duration is the time spent in an area   
 */
function formatData(activity) {
    // create color scale
    var colorScale = colorScheme(); 
    
    clickHeatmapData = new Array();
    moveHeatmapData = new Array();

    var maxClick = 0;
    var maxMove = 0;
    for(ix in activity) {
        if (activity[ix].evt === 'move') {    
            if (parseInt(activity[ix].dur) > maxMove) {
                maxMove = parseInt(activity[ix].dur);
            }
        } else {
            if (parseInt(activity[ix].dur) > maxClick) {
                maxClick = parseInt(activity[ix].dur);
            }
        }
    }

    var colorRange = colorScale.length - 1;
    var gridCenter = Math.ceil(gridAreaSize / 2);
    var pageWidth = parseInt($("#wrapper").css("min-width"));

    // determine information to format the heat maps
    for(ix in activity) {        
        var deGridX = parseInt(activity[ix].X) * gridAreaSize;
        var Xcoord = getLeftCoordinate(activity[ix].grid, deGridX) - gridAreaSize;
        var Ycoord = (activity[ix].Y * gridAreaSize) - gridAreaSize;
        if ((Xcoord + gridAreaSize) > pageWidth) {
            pageWidth = Xcoord + gridAreaSize;
        }
        if (activity[ix].evt === 'move') {
            var idx = Math.round((colorRange / (maxMove / parseInt(activity[ix].dur))), Number);
            moveHeatmapData.push({ 
                    "X": Xcoord + gridCenter,
                    "Y": Ycoord + gridCenter,
                    "rgb": colorScale[idx]
                });
        } else {
            var idx = Math.round((colorRange / (maxClick / parseInt(activity[ix].dur))), Number);
            clickHeatmapData.push({
                    "X": Xcoord + gridCenter,
                    "Y": Ycoord + gridCenter,
                    "rgb": colorScale[idx],
                    "xPoints": getXPoints(Xcoord, Ycoord)
                });
        }
    }
    
    // prevent width of elements from moving on resize for proper overlay of heat map graphics
    if (pageWidth < parseInt($("#maincontent").css("width"))) {
        pageWidth = parseInt($("#maincontent").css("width"));
    }
    $("#wrapper")
        .css({
            'width': pageWidth,
            'min-width': pageWidth,
            'max-width': pageWidth
        });
    $("#overlay")
        .css({
            'width': (pageWidth + 5),
            'min-width': (pageWidth + 5),
            'max-width': (pageWidth +5)
        });   
}



/*
 *  getLeftCoordinate
 *      determine the actual left coordinate position based on the current
 *      page width settings from the original captured coordine based on the
 *      page section the event occurred.
 */
function getLeftCoordinate(grid, xCoordinate) {
    var leftCoordinate = 0;
    if (grid === "nav") {
        leftCoordinate = navLeft + xCoordinate;            
    } else if (grid === "ftr") {
        leftCoordinate = footerLeft + xCoordinate;
    } else {
        leftCoordinate = xCoordinate;
    }
    return leftCoordinate;
}



/*
 *  getXPoints
 *      Determine the left-top, left-bottom, right-top, right-bottom points
 *      in order to display an X to show where the user performed a mouse click.
 *      Center point of the X is the center point the graphic display and the
 *      center of the area the event occured.
 */
function getXPoints(xLeft, yTop) {
    var xPoints = new Object();
    
    xPoints.LT = new Object();
    xPoints.LT.x = xLeft - 2;
    xPoints.LT.y = yTop - 2;
    
    xPoints.LB = new Object();
    xPoints.LB.x = xLeft - 2;
    xPoints.LB.y = yTop + (gridAreaSize + 2);
    
    xPoints.RT = new Object();
    xPoints.RT.x = xLeft + (gridAreaSize + 2);
    xPoints.RT.y = yTop - 2;
    
    xPoints.RB = new Object();
    xPoints.RB.x = xLeft + (gridAreaSize + 2);
    xPoints.RB.y = yTop + (gridAreaSize + 2);
    
    return xPoints;
}



/*
 *  loadHeatMap
 *      add graphic elements depicting user activity
 *      Note:
 *            svg code was derived from a combination of examples along with trial and
 *            error testing.  I was able to come up with the idea of using 4 points of
 *            the graphic X (slash and backslash depicting a mouse click) to be used 
 *            to set the start and end X/Y coordiates on the svg line tag attributes.
 */
function loadHeatMap() {
    // build click heat map
    $("body").append("<div id='clickMap'></div>");
    $("#clickMap").hide();
    $("#clickMap")
	.css({
            'opacity' : .6,
            'position' : 'absolute',
            'top' : 55,
            'left' : 5,
            'width': '100%',
            'height': '100%',
            'z-index': 9000
        });
        
    var svgString = '<svg xmls="http://www.w3.org/2000/svg" version="1.1" height="100%" width="100%">';
    for (ix in clickHeatmapData) {
        svgString += '<circle cx="' + clickHeatmapData[ix].X +
                     '" cy="' + clickHeatmapData[ix].Y +
                     '" r="' + (Math.ceil(gridAreaSize / 2) + 6) +
                     '" stroke="black" stroke-width="1"' + 
                     ' fill="' + clickHeatmapData[ix].rgb +
                     '" />';
             
        //  create an X inside circle (backslash and slash)
        //  create the backslash line: start (left-top) end (right-bottom)
        var x1 = clickHeatmapData[ix].xPoints.LT.x;
        var y1 = clickHeatmapData[ix].xPoints.LT.y;
        var x2 = clickHeatmapData[ix].xPoints.RB.x;
        var y2 = clickHeatmapData[ix].xPoints.RB.y;
        var svgLine1 = '<line x1="' + x1 + '" y1="' + y1 + '" x2="' + x2 + '" y2="' + y2 + 
                       '" stroke="white" stroke-width="6" />';
        svgString += svgLine1;
        
        //  create the slash line: start (left-bottom) end (right-top)
        x1 = clickHeatmapData[ix].xPoints.LB.x;
        y1 = clickHeatmapData[ix].xPoints.LB.y;
        x2 = clickHeatmapData[ix].xPoints.RT.x;
        y2 = clickHeatmapData[ix].xPoints.RT.y;
        var svgLine2 = '<line x1="' + x1 + '" y1="' + y1 + '" x2="' + x2 + '" y2="' + y2 + 
                       '" stroke="white" stroke-width="6" />';
        svgString += svgLine2;
        
        //  create an inner X (backslash and slash)
        svgLine1 = svgLine1.replace('white', 'black');
        svgLine1 = svgLine1.replace('width="6', 'width="2');
        svgString += svgLine1;
        svgLine2 = svgLine2.replace('white', 'black');
        svgLine2 = svgLine2.replace('width="6', 'width="2');
        svgString += svgLine2;
    } 
    svgString += '</svg>';   
    $("#clickMap").html(svgString);
    
    
    // build mouse search heat map
    $("body").append("<div id='moveMap'></div>");
    $("#moveMap")
	.css({
            'opacity' : .6,
            'position' : 'absolute',
            'top' : 55,
            'left' : 5,
            'width': '100%',
            'height': '100%',
            'z-index': 9000
        });
        
    svgString = '<svg xmls="http://www.w3.org/2000/svg" version="1.1" height="100%" width="100%">';
    for (ix in moveHeatmapData) {
        svgString += '<circle cx="' + moveHeatmapData[ix].X +
                     '" cy="' + moveHeatmapData[ix].Y +
                     '" r="' + (Math.ceil(gridAreaSize / 2) + 3) +
                     '" fill="' + moveHeatmapData[ix].rgb +
                     '" /> ';
    } 
    svgString += '</svg>';   
    $("#moveMap").html(svgString);
}



/*
 *  colorScheme
 *      build RGB color scale array with RGB values 
 *      to be used for graphic values depicting duration
 *      - build the entire range between min and max
 *        points values where number direction changes:
 *          Red     Green   Blue
 *          0       0       255     zero value  (blue)
 *          0       255     255                 (aqua)
 *          0       255     0                   (green)
 *          255     255     0                   (yellow)
 *          255     0       0       max value   (red)
 */
function colorScheme() {    
    scale = new Array();
    
    var hexRed = 0;
    var hexGreen = -1;
    var hexBlue = 255;
    do {    
        if (hexRed === 0 && hexBlue > 0) {
            if (hexGreen < 255) {
                hexGreen++;
            } else {
                hexBlue--;
            }
        } else if (hexRed < 255) {
            hexRed++;
        } else {
            hexGreen--;
        }
        var rgb = "rgb(" + hexRed + ", " + hexGreen + ", " + hexBlue + ")";
        scale.push(rgb);
    } while (!(hexRed === 255 && hexGreen === 0 && hexBlue === 0));

    return scale;
}



/*
 *  preparePageforMap
 *      turn off all normal page operations
 *      add heat map processing and background elements
 */
function preparePageforMap() {
    $("body *").each(function() {
        $(this).css("z-index", -1);
    });
    $("#home").css("z-index", 10);
    $("#nav").css("z-index", 10);
    
    // shift page view down allowing for placement of heat map buttons
    $("#wrapper").css('top', '50px');
    
    // create heat map overlay
    $("body").append("<div id='overlay'></div>");
    $("#overlay")
	.height($("#wrapper").height())
	.css({
            'opacity': 0.3,
            'position': 'absolute',
            'top': 55,
            'left': 5,
            'background-color': 'black',
            'z-index': 100
    });
    
    // add button to remove heat map and return to normal page operations
    $("body").append('<button type="button" id="heatmapoffbtn">Turn Off HeatMap</button>');
    $("#heatmapoffbtn")
	.css({ 
            'position': 'absolute',
            'top': 20,
            'left': 20
        })
	.on ('click', resetPage);
    
    // add button to toggle between mouse search heatmap and click heatmap
    $("body").append('<button type="button" id="switchmaps">View Click Activity</button>');
    $("#switchmaps")
	.css({ 
            'position': 'absolute',
            'top': 20,
            'left': 300
        })
	.on ('click', switchMap); 
}



/*
 *  switchMap
 *      toggle mousemove heatmap and click heatmap
 */
function switchMap() {
    if ($("#switchmaps").html() === "View Click Activity") {
        $("#moveMap").hide();
        $("#clickMap").show();
        $("#switchmaps").html("View Mouse Search Activity");
    } else {
        $("#clickMap").hide();
        $("#moveMap").show();
        $("#switchmaps").html("View Click Activity");
    }
}



/*
 *  resetPage
 *      remove heat map elements
 *      reset page elements for normal operations
 */
function resetPage() {
    // remove heat map elements
    $("#heatmapoffbtn").remove();
    $("#switchmaps").remove();
    $("#resetDB").remove();
    
    $("#moveMap").remove();
    $("#clickMap").remove();
    $("#overlay").remove();
    
    // reset page elements
    $("#wrapper")
        .css({
            'top': 'auto',
            'width': '100%',
            'min-width': 580,
            'max-width': 'none'
    });
    $("body *").each(function() {
        $(this).css("z-index", 'auto');
    });
    
    // window may have been resized, ensure page section variables are correct
    pageGridSections();
            
    // clear last entry (only one will exist being the intial time capture of the 
    // mouse moving to the heatmap button).  If not done, the next mousemove event
    // will calculate the duration based on the time spent viewing the heat map.
    sessionStorage.removeItem(moveStorageKey);
    
    // turn on normal page operations
    $(window).on("resize", pageGridSections);
    $("#wrapper").on("click", captureMouseClick);
    $("#wrapper").on("mousemove", captureMouseMovement);
    $("#heatmapbtn").attr("disabled", false);
    timer = setInterval(sendDataToServer, 10000);
}



/*
 *  pageGridSections
 *      Determine the current position of the nav and footercontent divs.
 *      These html divs move when the page is resized.
 *      Capturing heat map data activity based on rendered position will
 *      allow propper display of activity when viewing the heat map.
 */
function pageGridSections() {
    // nav div in header (positioned on right of page)
    navTop = parseInt($("#nav").css("top"));
    navBottom = navTop + parseInt($("#nav").css("height"));
    if ($("#nav").css("left") === "auto") {
        navLeft = parseInt($("#wrapper").css("width"))
                        - parseInt($("#nav").css("width"))
                        - parseInt($("#nav").css("right"))
                        + parseInt($("#nav").css("margin-left"));
                        - parseInt($("#nav").css("margin-right"));
    } else {
        navLeft = parseInt($("#nav").css("left")) + parseInt($("#nav").css("margin-left"));
    }
    navRight = parseInt($("#wrapper").css("width")) - parseInt($("#nav").css("margin-right"));

    // footercontent div (centered in page)
    footerBottom = parseInt($("#wrapper").css("height"));
    footerTop = footerBottom - parseInt($("#footercontent").css("height"));
    var footerWidth = parseInt($("#footerlinks").css("width"));
    footerWidth += parseInt($("#footerlinks").css("padding-left")) + parseInt($("#footerlinks").css("padding-right"));
    footerLeft = (parseInt($("#wrapper").css("width")) / 2) - (footerWidth / 2);
    footerRight = footerLeft + footerWidth;
}

/*
 *      End of Script
 */
